class MonsterList:
    monsters = [
        {
            "id": 1,
            "name": "Goblin",
            "level": 1,
            "hd": 4,
            "atk": 2,
            "ac": 2,
            "total": 8,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 2,
            "name": "Kobold",
            "level": 1,
            "hd": 3,
            "atk": 3,
            "ac": 1,
            "total": 7,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "pierce",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 3,
            "name": "Dog",
            "level": 1,
            "hd": 2,
            "atk": 4,
            "ac": 2,
            "total": 8,
            "type": "animal",
            "subtype": "mammal",
            "atk_type": "pierce",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 4,
            "name": "Jelly",
            "level": 1,
            "hd": 5,
            "atk": 1,
            "ac": 4,
            "total": 10,
            "type": "aberration",
            "subtype": "",
            "atk_type": "acid",
            "resist": "acid",
            "vulnerability": "electric",
            "special": "melt"
        },
        {
            "id": 5,
            "name": "Skeleton",
            "level": 2,
            "hd": 3,
            "atk": 4,
            "ac": 3,
            "total": 10,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "slash",
            "resist": "slash",
            "vulnerability": "blunt",
            "special": ""
        },
        {
            "id": 6,
            "name": "Orc",
            "level": 2,
            "hd": 4,
            "atk": 3,
            "ac": 3,
            "total": 10,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "blunt",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 7,
            "name": "Slime",
            "level": 2,
            "hd": 7,
            "atk": 3,
            "ac": 5,
            "total": 15,
            "type": "aberration",
            "subtype": "",
            "atk_type": "acid",
            "resist": "acid",
            "vulnerability": "electric",
            "special": "melt"
        },
        {
            "id": 8,
            "name": "Wolf",
            "level": 2,
            "hd": 3,
            "atk": 4,
            "ac": 3,
            "total": 10,
            "type": "animal",
            "subtype": "mammal",
            "atk_type": "pierce",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 9,
            "name": "Imp",
            "level": 2,
            "hd": 3,
            "atk": 5,
            "ac": 3,
            "total": 11,
            "type": "demon",
            "subtype": "",
            "atk_type": "fire",
            "resist": "fire",
            "vulnerability": "cold",
            "special": "burn"
        },
        {
            "id": 10,
            "name": "Gnoll",
            "level": 3,
            "hd": 4,
            "atk": 4,
            "ac": 3,
            "total": 11,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "blunt",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 11,
            "name": "Zombie",
            "level": 3,
            "hd": 7,
            "atk": 3,
            "ac": 5,
            "total": 15,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "blunt",
            "resist": "cold",
            "vulnerability": "fire",
            "special": ""
        },
        {
            "id": 12,
            "name": "Python",
            "level": 3,
            "hd": 5,
            "atk": 5,
            "ac": 5,
            "total": 15,
            "type": "animal",
            "subtype": "reptile",
            "atk_type": "grab",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 13,
            "name": "Spectre",
            "level": 4,
            "hd": 6,
            "atk": 6,
            "ac": 4,
            "total": 16,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "touch",
            "resist": "",
            "vulnerability": "",
            "special": "drain"
        },
        {
            "id": 14,
            "name": "Griffon",
            "level": 4,
            "hd": 5,
            "atk": 7,
            "ac": 4,
            "total": 16,
            "type": "animal",
            "subtype": "",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 15,
            "name": "Ogre",
            "level": 4,
            "hd": 7,
            "atk": 7,
            "ac": 5,
            "total": 19,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "blunt",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 16,
            "name": "Quasit",
            "level": 4,
            "hd": 4,
            "atk": 5,
            "ac": 7,
            "total": 16,
            "type": "demon",
            "subtype": "",
            "atk_type": "pierce",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 17,
            "name": "Ooze",
            "level": 4,
            "hd": 8,
            "atk": 3,
            "ac": 5,
            "total": 16,
            "type": "aberration",
            "subtype": "",
            "atk_type": "acid",
            "resist": "electric",
            "vulnerability": "fire",
            "special": "melt"
        },
        {
            "id": 18,
            "name": "Spider",
            "level": 5,
            "hd": 6,
            "atk": 8,
            "ac": 7,
            "total": 21,
            "type": "animal",
            "subtype": "insect",
            "atk_type": "pierce",
            "resist": "pierce",
            "vulnerability": "fire",
            "special": "drain"
        },
        {
            "id": 19,
            "name": "Shambler",
            "level": 5,
            "hd": 7,
            "atk": 8,
            "ac": 5,
            "total": 20,
            "type": "aberration",
            "subtype": "",
            "atk_type": "blunt",
            "resist": "fire",
            "vulnerability": "slash",
            "special": ""
        },
        {
            "id": 20,
            "name": "Wraith",
            "level": 5,
            "hd": 5,
            "atk": 8,
            "ac": 11,
            "total": 24,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "cold",
            "resist": "cold",
            "vulnerability": "electric",
            "special": "drain"
        },
        {
            "id": 21,
            "name": "Werewolf",
            "level": 5,
            "hd": 6,
            "atk": 8,
            "ac": 7,
            "total": 21,
            "type": "animal",
            "subtype": "mammal",
            "atk_type": "slash",
            "resist": "blunt",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 22,
            "name": "Drake",
            "level": 5,
            "hd": 9,
            "atk": 8,
            "ac": 9,
            "total": 26,
            "type": "dragon",
            "subtype": "",
            "atk_type": "slash",
            "resist": "fire",
            "vulnerability": "cold",
            "special": "burn"
        },
        {
            "id": 23,
            "name": "Gremlin",
            "level": 6,
            "hd": 7,
            "atk": 9,
            "ac": 8,
            "total": 24,
            "type": "demon",
            "subtype": "",
            "atk_type": "acid",
            "resist": "fire",
            "vulnerability": "blunt",
            "special": "melt"
        },
        {
            "id": 24,
            "name": "Panther",
            "level": 6,
            "hd": 6,
            "atk": 11,
            "ac": 9,
            "total": 26,
            "type": "animal",
            "subtype": "",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 25,
            "name": "Automaton",
            "level": 6,
            "hd": 10,
            "atk": 8,
            "ac": 8,
            "total": 26,
            "type": "humanoid",
            "subtype": "construct",
            "atk_type": "electric",
            "resist": "blunt",
            "vulnerability": "pierce",
            "special": "shock"
        },
        {
            "id": 26,
            "name": "Bugbear",
            "level": 6,
            "hd": 8,
            "atk": 10,
            "ac": 9,
            "total": 27,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "blunt",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 27,
            "name": "Crab",
            "level": 7,
            "hd": 8,
            "atk": 10,
            "ac": 12,
            "total": 30,
            "type": "animal",
            "subtype": "insect",
            "atk_type": "slash",
            "resist": "slash",
            "vulnerability": "cold",
            "special": ""
        },
        {
            "id": 28,
            "name": "Lizardman",
            "level": 7,
            "hd": 10,
            "atk": 10,
            "ac": 10,
            "total": 30,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "pierce",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 29,
            "name": "Ghoul",
            "level": 7,
            "hd": 9,
            "atk": 11,
            "ac": 11,
            "total": 31,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "slash",
            "resist": "cold",
            "vulnerability": "fire",
            "special": "drain"
        },
        {
            "id": 30,
            "name": "Gargoyle",
            "level": 7,
            "hd": 12,
            "atk": 8,
            "ac": 10,
            "total": 30,
            "type": "humanoid",
            "subtype": "construct",
            "atk_type": "slash",
            "resist": "blunt",
            "vulnerability": "cold",
            "special": ""
        },
        {
            "id": 31,
            "name": "Minotaur",
            "level": 8,
            "hd": 8,
            "atk": 11,
            "ac": 9,
            "total": 34,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 32,
            "name": "Bear",
            "level": 8,
            "hd": 9,
            "atk": 11,
            "ac": 5,
            "total": 31,
            "type": "animal",
            "subtype": "",
            "atk_type": "grab",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 33,
            "name": "Golem",
            "level": 8,
            "hd": 10,
            "atk": 10,
            "ac": 10,
            "total": 36,
            "type": "humanoid",
            "subtype": "construct",
            "atk_type": "blunt",
            "resist": "slash",
            "vulnerability": "electric",
            "special": ""
        },
        {
            "id": 34,
            "name": "Shadow",
            "level": 8,
            "hd": 7,
            "atk": 13,
            "ac": 7,
            "total": 33,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "touch",
            "resist": "electric",
            "vulnerability": "fire",
            "special": "drain"
        },
        {
            "id": 35,
            "name": "Centaur",
            "level": 9,
            "hd": 11,
            "atk": 13,
            "ac": 12,
            "total": 39,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 36,
            "name": "Eagle",
            "level": 9,
            "hd": 10,
            "atk": 15,
            "ac": 13,
            "total": 41,
            "type": "animal",
            "subtype": "bird",
            "atk_type": "pierce",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 37,
            "name": "Yeti",
            "level": 9,
            "hd": 14,
            "atk": 12,
            "ac": 12,
            "total": 41,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "blunt",
            "resist": "cold",
            "vulnerability": "fire",
            "special": "freeze"
        },
        {
            "id": 38,
            "name": "Wyvern",
            "level": 9,
            "hd": 12,
            "atk": 14,
            "ac": 13,
            "total": 42,
            "type": "dragon",
            "subtype": "",
            "atk_type": "pierce",
            "resist": "",
            "vulnerability": "",
            "special": "drain"
        },
        {
            "id": 27,
            "name": "Tiger",
            "level": 10,
            "hd": 11,
            "atk": 13,
            "ac": 15,
            "total": 30,
            "type": "animal",
            "subtype": "",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 28,
            "name": "Hag",
            "level": 10,
            "hd": 13,
            "atk": 13,
            "ac": 13,
            "total": 30,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "fire",
            "resist": "fire",
            "vulnerability": "pierce",
            "special": "burn"
        },
        {
            "id": 29,
            "name": "Will-o-wisp",
            "level": 10,
            "hd": 12,
            "atk": 14,
            "ac": 14,
            "total": 31,
            "type": "humanoid",
            "subtype": "fae",
            "atk_type": "electric",
            "resist": "electric",
            "vulnerability": "blunt",
            "special": "shock"
        },
        {
            "id": 30,
            "name": "Hell Hound",
            "level": 10,
            "hd": 15,
            "atk": 11,
            "ac": 13,
            "total": 30,
            "type": "demon",
            "subtype": "",
            "atk_type": "fire",
            "resist": "fire",
            "vulnerability": "cold",
            "special": "burn"
        },
        {
            "id": 31,
            "name": "Scorpion",
            "level": 11,
            "hd": 12,
            "atk": 15,
            "ac": 13,
            "total": 34,
            "type": "animal",
            "subtype": "insect",
            "atk_type": "pierce",
            "resist": "slash",
            "vulnerability": "blunt",
            "special": "drain"
        },
        {
            "id": 32,
            "name": "Mummy",
            "level": 11,
            "hd": 13,
            "atk": 15,
            "ac": 9,
            "total": 31,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "blunt",
            "resist": "cold",
            "vulnerability": "fire",
            "special": "drain"
        },
        {
            "id": 33,
            "name": "Tentaculus",
            "level": 11,
            "hd": 14,
            "atk": 14,
            "ac": 14,
            "total": 36,
            "type": "aberration",
            "subtype": "",
            "atk_type": "grab",
            "resist": "blunt",
            "vulnerability": "slash",
            "special": "melt"
        },
        {
            "id": 34,
            "name": "Troll",
            "level": 11,
            "hd": 11,
            "atk": 17,
            "ac": 11,
            "total": 33,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "slash",
            "resist": "pierce",
            "vulnerability": "fire",
            "special": ""
        },
        {
            "id": 35,
            "name": "Naga",
            "level": 12,
            "hd": 12,
            "atk": 14,
            "ac": 13,
            "total": 39,
            "type": "demon",
            "subtype": "",
            "atk_type": "grab",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 36,
            "name": "Chimera",
            "level": 12,
            "hd": 11,
            "atk": 16,
            "ac": 14,
            "total": 41,
            "type": "animal",
            "subtype": "",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 37,
            "name": "Vampire",
            "level": 12,
            "hd": 15,
            "atk": 13,
            "ac": 13,
            "total": 41,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "pierce",
            "resist": "electric",
            "vulnerability": "fire",
            "special": "drain"
        },
        {
            "id": 38,
            "name": "Manticore",
            "level": 12,
            "hd": 13,
            "atk": 15,
            "ac": 14,
            "total": 42,
            "type": "animal",
            "subtype": "",
            "atk_type": "pierce",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 39,
            "name": "Ghost",
            "level": 13,
            "hd": 13,
            "atk": 15,
            "ac": 17,
            "total": 45,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "touch",
            "resist": "slash",
            "vulnerability": "",
            "special": "drain"
        },
        {
            "id": 40,
            "name": "Giant",
            "level": 13,
            "hd": 15,
            "atk": 15,
            "ac": 15,
            "total": 45,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "blunt",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 41,
            "name": "Wyrm",
            "level": 13,
            "hd": 14,
            "atk": 16,
            "ac": 16,
            "total": 46,
            "type": "dragon",
            "subtype": "",
            "atk_type": "fire",
            "resist": "pierce",
            "vulnerability": "cold",
            "special": "burn"
        },
        {
            "id": 42,
            "name": "Horror",
            "level": 13,
            "hd": 17,
            "atk": 13,
            "ac": 15,
            "total": 45,
            "type": "aberration",
            "subtype": "",
            "atk_type": "grab",
            "resist": "fire",
            "vulnerability": "electric",
            "special": "drain"
        },
        {
            "id": 43,
            "name": "Death Knight",
            "level": 14,
            "hd": 15,
            "atk": 18,
            "ac": 16,
            "total": 49,
            "type": "humanoid",
            "subtype": "undead",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": "freeze"
        },
        {
            "id": 44,
            "name": "Hydra",
            "level": 14,
            "hd": 16,
            "atk": 18,
            "ac": 12,
            "total": 46,
            "type": "animal",
            "subtype": "reptile",
            "atk_type": "pierce",
            "resist": "slash",
            "vulnerability": "fire",
            "special": ""
        },
        {
            "id": 45,
            "name": "Titan",
            "level": 14,
            "hd": 17,
            "atk": 17,
            "ac": 17,
            "total": 51,
            "type": "humanoid",
            "subtype": "",
            "atk_type": "slash",
            "resist": "",
            "vulnerability": "",
            "special": ""
        },
        {
            "id": 46,
            "name": "Demon",
            "level": 14,
            "hd": 14,
            "atk": 20,
            "ac": 14,
            "total": 48,
            "type": "demon",
            "subtype": "",
            "atk_type": "pierce",
            "resist": "fire",
            "vulnerability": "acid",
            "special": "burn"
        }
    ]

    descriptors = {
        "humanoid" : [
            "Big",
            "Burly",
            "Massive",
            "Hulking",
            "Monstrous"
        ],
        "animal" : [
            "Wild",
            "Snarling",
            "Vicious",
            "Rabid",
            "Dire"
        ],
        "reptile" : [
            "Hissing",
            "Lurking",
            "Vicious",
            "Striking",
            "Dire"
        ],
        "insect" : [
            "Shiny",
            "Chittering",
            "Gangly",
            "Chitinous",
            "Looming"
        ],
        "undead" : [
            "Spooky",
            "Moaning",
            "Rotting",
            "Wailing",
            "Eternal"
        ],
        "fae" : [
            "Glittering",
            "Glamoured",
            "Forest",
            "Ancient",
            "First"
        ],
        "construct" : [
            "Wooden",
            "Clay",
            "Stone",
            "Iron",
            "Crystal"
        ],
        "aberration" : [
            "Weird",
            "Strange",
            "Confusing",
            "Horrifying",
            "Insane"
        ],
        "demon" : [
            "Smoking",
            "Glowing",
            "Flaming",
            "Inferno",
            "Holocaust"
        ],
        "dragon" : [
            "Young",
            "Mature",
            "Gleaming",
            "Ancent",
            "Elder"
        ]
    }

    bossDescriptors = {
        "humanoid" : [
            ("", "King"),
            ("Mighty", "Warrior"),
            ("Ultimate", "")
        ],
        "animal" : [
            ("Prehistoric", ""),
            ("Legendary", ""),
            ("", "of the Wildlands")
        ],
        "reptile" : [
            ("Enormous", ""),
            ("Cthonic", ""),
            ("Fanged", "")
        ],
        "insect" : [
            ("", "Queen"),
            ("Emerald", ""),
            ("Winged", "")
        ],
        "undead" : [
            ("", "Lord"),
            ("Unholy", ""),
            ("", "of Nightmares")
        ],
        "fae" : [
            ("", "Noble"),
            ("Unseelie", ""),
            ("", "of the Deep Green")
        ],
        "aberration" : [
            ("Royal", ""),
            ("Great Old", ""),
            ("", "That Never Sleeps")
        ],
        "demon" : [
            ("Noble", "of Dis"),
            ("Damned", ""),
            ("", "Torturer")
        ],
        "dragon" : [
            ("Mother", ""),
            ("Golden-Winged", ""),
            ("Black", "of the Pit")
        ]
    }

    bossQuotes = {
        "humanoid" : [
            '"You\'ll never survive me!"',
            '"I\'m your worst nightmare!"',
            '"Come a little closer..."'
        ],
        "animal" : [
            "It roars a challenge!",
            "The ground shakes as it stomps toward you!",
            "You feel its eyes on you..."
        ],
        "insect" : [
            "Venom drips from its fangs!",
            "It chitters menacingly...",
            "It gathers itself to strike..."
        ],
        "undead" : [
            '"Join me in the world beyond life!"',
            "Its eyes glow with a cold light...",
            '"Life is wasted on the living!"'
        ],
        "fae" : [
            'It whispers eldritch words you half understand...',
            "You feel drawn toward another realm...",
            'It suddenly appears right next to you!'
        ],
        "construct" : [
            'Gears grind within its body...',
            "Lifeless eyes track your movement...",
            'It relentlessly pursues you!'
        ],
        "aberration" : [
            "It contorts weirdly...",
            "You see unnatural shapes in its surface...",
            "It surges toward you!"
        ],
        "demon" : [
            '"The underworld awaits!"',
            '"Time to pay for your sins!"',
            '"Feel the flames of the Abyss!"'
        ],
        "dragon" : [
            '"You look tasty..."',
            "It coils itself, ready to attack!",
            '"My treasures are mine alone!"'
        ]
    }

    atkVerbs = {
        "slash": [
            "slashes",
            "cuts",
            "hacks"
        ],
        "pierce": [
            "stabs",
            "pokes",
            "impales"
        ],
        "blunt": [
            "bashes",
            "smashes",
            "crushes"
        ],
        "touch": [
            "touches",
            "grips",
            "gropes"
        ],
        "grab": [
            "grabs",
            "squeezes",
            "crushes"
        ],
        "fire": [
            "scorches",
            "burns",
            "chars"
        ],
        "cold": [
            "freezes",
            "chills",
            "frosts"
        ],
        "electric": [
            "zaps",
            "shocks",
            "jolts"
        ],
        "acid": [
            "dissolves",
            "burns",
            "melts"
        ],
        "animal": {
            "slash": [
                "claws",
                "rakes",
                "slashes"
            ],
            "pierce": [
                "bites",
                "gnaws",
                "chews"
            ]
        },
        "aberration": {
            "grab": [
                "engulfs",
                "smothers",
                "grapples"
            ],
            "blunt": [
                "slams",
                "slaps",
                "mangles"
            ]
        },
        "demon": {},
        "dragon": {
            "slash": [
                "claws",
                "rakes",
                "slashes"
            ],
            "pierce": [
                "impales",
                "stings",
                "bites"
            ]
        },
        "insect": {
            "pierce": [
                "impales",
                "stings",
                "bites"
            ]
        }
    }


class RoomList:
    descriptor_types = [
        "quality",
        "size",
        "shape",
        "color",
        "material",
        "purpose"
    ]

    descriptors = {
        "quality": [
            "musty",
            "dusty",
            "damp",
            "hot",
            "cold",
            "dark"
        ],
        "size": [
            "tiny",
            "small",
            "big",
            "huge",
            "enormous"
        ],
        "shape": [
            "round",
            "square",
            "oval",
            "pentagonal",
            "jagged"
        ],
        "color": [
            "grey",
            "black",
            "red",
            "green",
            "blue",
            "silver",
            "gold",
            "purple"
        ],
        "material": [
            "stone",
            "wood",
            "brick",
            "crystal",
            "metal"
        ],
        "purpose": [
            "torture",
            "guard",
            "dining",
            "throne",
            "trophy"
        ] 
    }


import random

class Door:
    def __init__(self, doorType, data = None):
        if data:
            for k in data:
                setattr(self, k, data[k])
        else:
            self.type = doorType
            self.exists = random.choice([True, True, False])
            self.seen = False
            self.used = False

    def printStats(self):
        print("<< Door >>")

    def useDoor(self):
        self.used = True

    def isValid(self):
        return True if self.type == "stairs" else self.exists

    def printMap(self):
        if self.exists and self.seen:
            return "==" if self.type == "ew" else " | "
        else:
            return "  " if self.type == "ew" else "   "



import random
import copy

from colored import fore, back, style

from .creature import Creature
from .item import Item
from .utils import Utils
from .item_list import ItemList

class Player(Creature):
    def __init__(self, name, saveInfo = None):
        self.level = 1
        self.xp = 0
        self.gp = 0
        self.hp = 0
        self.ac = 10
        self.maxHp = 0
        self.history = {}
        self.name = name
        self.nextLevel = 100
        self.items = []
        self.skills = []
        self.innateAbilities = {}
        self.abilities = {}
        self.conditions = {}
        self.resist = []
        self.monsterLore = {}
        self.hasIdol = False
        self.saveId = ""
        
        info = saveInfo if saveInfo else {
            "name": name,
            "level": 1,
            "nextLevel": 1000,
            "atk": 1,
            "hp": 10,
            "maxHp": 10,
            "atkType": 'blunt',
            "history": {
                "rest": 0,
                "risky_win": 0,
                "reckless": 0,
                "run_away": 0,
                "kills": 0,
                "buy_item": 0,
                "sell_item": 0,
                "dmg_done": 0,
                "dmg_taken": 0,
                "epitaph": "Still exploring..."
            }
        }
        Creature.__init__(self, info)

    def loadItems(self, itemData):
        self.items = []
        for data in itemData:
            self.items.append(Item(0, data))

    def addGold(self, value):
        self.gp += value
        
    def removeGold(self, value):
        self.gp -= value

    def addItem(self, item):
        # auto-equip item if no item of this type is equipped
        kind = item.kind
        if kind == "usable":
            found = False
            for invItem in self.items:
                if invItem.id == item.id:
                    found = True
                    invItem.stack += 1
            if not found:
                self.items.append(item)
        else:
            found = False
            for invItem in self.items:
                if invItem.kind == kind:
                    found = True
            if not found:
                item.equipped = True
            self.items.append(item)
            self.applyItems()
        
    def removeItem(self, item):
        if item.kind == "usable" and item.stack > 1:
            item.stack -= 1
        else:
            self.items.remove(item)
        self.applyItems()
        
    def equipItem(self, newItem):
        for invItem in self.items:
            if invItem.kind == newItem.kind and invItem.equipped:
                invItem.equipped = False
        newItem.equipped = True

        self.applyItems()
        
    def unequipItem(self, kind):
        for invItem in self.items:
            if invItem.kind == kind and invItem.equipped:
                invItem.equipped = False
        self.applyItems()
        
    def applyItems(self):
        self.atk = (self.level + 1) // 2
        self.ac = 10
        self.abilities = copy.copy(self.innateAbilities)
        self.resist = []
        self.atkType = "blunt"

        for item in self.items:
            if item.equipped:
                if item.atk:
                    self.atk += item.atk
                if item.ac:
                    self.ac += item.ac
                if item.kind == "weapon":
                    self.atkType = item.type
                if item.ability:
                    if "resist_" in item.ability:
                        self.resist.append(item.ability.replace("resist_", ""))
                    else:
                        try: 
                            self.abilities[item.ability] += item.getAbilityLevel()
                        except KeyError:
                            self.abilities[item.ability] = item.getAbilityLevel()

        super().calculateDam()

    def checkLevelUp(self):
        if self.xp >= self.nextLevel:
            self.level += 1
            self.hp += 10
            self.maxHp += 10
            self.nextLevel += 1000 * self.level
            self.applyItems()
            return True
        else:
            return False

    def getAbilityLevel(self, ability):
        level = 0
        try:
            level = self.abilities[ability]
        except KeyError:
            pass
        return level

    def getAtkVerb(self):
        verbs = ItemList.atkVerb[self.atkType]
        return random.choice(verbs)
        
    def incrementHistory(self, field, value = 1):

        self.history[field] += value

    def drain(self, value):
        self.xp -= value
        if self.xp < 0:
            self.xp = 0
            
    def killedBy(self, monster, level):
        self.setEpitaph(f"Killed by a {monster.name} on level {level}.")

    def setEpitaph(self, text):
        self.history["epitaph"] = text

    def printStats(self):
        print(f"{fore.MAGENTA}{style.BOLD}{self.name}{style.RESET}")

        hpColor = style.RESET
        if self.hp / self.maxHp <= .25:
            hpColor = fore.RED
        elif self.hp / self.maxHp <= .6:
            hpColor = fore.YELLOW

        stats = [
            {
                "Level": f"{self.level}",
                "XP": f"{self.xp} / {self.nextLevel}",
                "GP": f"{self.gp}"
            },
            {
                "HP": f"{hpColor}{self.hp}{style.RESET} / {self.maxHp}",
                "ATK": f"{self.atk} {Creature.calculateDam(self)}",
                "AC": f"{self.ac}",
            }
        ]
        Utils.printStats(stats)
        
        itemNames = []
        if len(self.items) == 0:
            itemNames = ["None"]
        for item in self.items:
            if item.equipped:
                itemNames.append(item.displayName)
        print("Equipped: " + ", ".join(itemNames))

    def getLoreRating(self, floor):
        maxLore = (floor + 1) * 12
        count = 0
        for key, m in self.monsterLore.items():
            if m["resist"]:
                count += 1
            if m["vulnerability"]:
                count += 1
            if m["special"]:
                count += 1
        return count / maxLore * 100

    def hasSeenMonster(self, monster):
        try:
            lore = self.monsterLore[monster.id]
            return True
        except KeyError:
            return False

    def printHistory(self, turns):
        print(f"{fore.MAGENTA}{style.BOLD}{self.name}{style.RESET}")
        print(f"{fore.RED}{self.history['epitaph']}{style.RESET}\n")

        stats = [
            { 
                "Total Turns": f"{turns}",
                "Times Rested": f"{self.history['rest']}",
                "Battles Fled": f"{self.history['run_away']}"
            },
            {
                "Kills": f"{self.history['kills']}",
                "Reckless Attacks": f"{self.history['reckless']}",
                "Risky Wins": f"{self.history['risky_win']}" 
            },
            { 
                "Items Bought": f"{self.history['buy_item']}",
                "Items Sold": f"{self.history['sell_item']}" 
            },
            { 
                "Damage Done": f"{self.history['dmg_done']}",
                "Damage Taken": f"{self.history['dmg_taken']}" 
            },
        ]
        Utils.printStats(stats)
